# File: storebox.py
# Purpose: General purpose message storage box

import cPickle
import string
import gtk
import boxformats
import boxformats.configbox
from pyneheaders import *
from superbox import *
from ptk.big_edit_box import *
import ptk.misc_widgets

class storebox(superbox):
	"""
	Simply stores messages with no special send, receive or delete rules.
	Typical use as a 'sent' or 'saved' folder.
	"""
	def get_flags(self):
		# special ones may not be deleted
		if self.uid in ("sent", "deleted"):
			return BOX_DEL_MESSAGE | BOX_IMPORT_TO
		else:
			return BOX_ISDELETEABLE | BOX_DEL_MESSAGE | \
			       BOX_IMPORT_TO

	def __init__(self, user, uid):
		self.load(user, uid)
	
	def save(self, user):

		tosave = {}

		def _saveme(key, tosave=tosave, self=self):
			tosave[key] = self.__dict__[key]

		_saveme("uid")
		_saveme("name")
		_saveme("opts")
		_saveme("expire_after")
		_saveme("export_format")
		_saveme("num_unread")

		f = open(self.uid+".fol", "w")

		cPickle.dump(ver_stamp, f, 1)
		cPickle.dump("STOREBOX", f, 1)
		cPickle.dump(tosave, f, 1)

		f.close()

	def load(self, user, uid):
		self.load_box (user, uid)

		def _ifmissing(key, value, self=self):
			if not self.__dict__.has_key(key):
				self.__dict__[key] = value

		_ifmissing("uid", uid)
		_ifmissing("name", _("Folder"))
		_ifmissing("opts", 0)
		_ifmissing("expire_after", None)
		_ifmissing("export_format", (boxformats.BF_FLATFILEBOX, None))
		_ifmissing("num_unread", 0)
		
		self.contents = []
		self.startup(user)

	def startup(self, user):
		"""
		Get an io module for loading/saving articles
		"""
		i = self.export_format[0]
		# this is a pretty piece of code :-)
		bf_class = boxformats.__dict__[boxformats.__all__[i]]. \
			   __dict__[boxformats.__all__[i]]
		self._io = bf_class(user, self)
		self.messages = self._io.get_contents()

	def get_menu(self):
		"""
		This box type specific options.
		"""
		return []

	def menu_chosen(self, _a, choice):
		"""
		Activate the chosen menu option.
		"""
		pass

	def getstats(self):
		"""
		Returns a tuple. (total msgs,).
		"""
		return (str (self.num_unread) + " ("+ str(len(self.messages)) + ")",)

	def setup(self, user, parent_win):
		"""
		Configure a sent mail box.
		"""
		win = gtk.Window()
		win.set_title(_("%s Folder Setup") % self.name)
		win.set_transient_for(parent_win)

		box = gtk.VBox()
		win.add(box)
		box.show()

		notebook = gtk.Notebook()
		notebook.set_tab_pos(gtk.POS_TOP)
		box.pack_start(notebook)
		notebook.show()

		# first page: misc settings
		settings_box = big_edit_box(self, 
		      ( ("name", _("Name:"), VAR_TYPE_STRING, 0, 0),
			("expire_after", _("Expire after (days):"), VAR_TYPE_INTEGER, 0, None),
			("opts", _("Thread messages"), VAR_TYPE_PACKED_BIT_BOOLEAN, 0, OPT_THREAD) )
		)
		label = gtk.Label(_("Settings"))
		notebook.append_page(settings_box, label)
		settings_box.show()

		# 2nd page: storage method
		label = gtk.Label(_("Mailbox Format"))
		formatbox = boxformats.configbox.configbox(self, user, win)
		notebook.append_page(formatbox, label)
		formatbox.show()

		def save_changes(_button, self=self, user=user, win=win, \
				settings_box=settings_box):
			# Extract info
			settings_box.apply_changes()
			# save changes to disk
			self.save(user)
			# update folder list
			self.changed = 1
			user.update()

		def save_changes_close(_button, win=win, save_changes=save_changes):
			save_changes(_button)
			win.destroy()

		def _cancel(_button, win=win):
			win.destroy()

		# Buttons at bottom
		buttonbox = ptk.misc_widgets.MyButtonBox()
		box.pack_start(buttonbox, expand=False)
		buttonbox.show()

		button = gtk.Button(stock="gtk-ok")
		button.connect("clicked", save_changes_close)
		buttonbox.pack_end(button, expand=False)
		button.show()

		button = gtk.Button(stock="gtk-apply")
		button.connect("clicked", save_changes)
		buttonbox.pack_end(button, expand=False)
		button.show()

		button = gtk.Button(stock="gtk-cancel")
		button.connect("clicked", _cancel)
		buttonbox.pack_end(button, expand=False)
		button.show()

		win.show()

